/*******************************************************************
*
*       Simulated Maximum Likelihood estimation of the
*       stochastic volatiliy model
*       version 1.2
*       February, 1996
*
*       Jon Danielsson (c)
*       Department of Economics
*       University of Iceland
*       101 Reykjavik
*       Iceland
*       e-mail: jond@hag.hi.is
*       ftp  hag.hi.is  
*       WWW  http://www.hag.hi.is/~jond
*       
*       please cite in published work and inform the author.
*
******************************************************************/


#include "svm.h"
#include "common.h"

int command_line::parse(int argc, char *argv[])
{
	int i;
//	Set up defaults 
	N	= 25;		// Number of simulations
	H	= YES;		// Get Hessian matrix (for s.e.'s)
	I	= 30;		// Iterations on the AGIS algorithm
	P	= 1e-4;		// Optimization Precision
	Z	= 1e-6;		// Hessian Precision
	Seed	= 234.0;	// Seed for RNG
	Print	= 1;		// What to output
				// Initial parameter values
	omega	= 0.0;
	delta 	= 0.95;
	nu	= 0.4;

	Ascale	= YES;		// Autoscale 
	Reg 	= NO;		// Do Fast regression

	if(argc < 2) 
   	{
		cout << "\nSimulated Maximum Likelihood estimation of the ";
		cout << "\nStochastic Volatity Model";
		cout << "\nVersion " 
			<< SVM_VERSION << "  " << SVM_DATE << endl;
		cout << "\n (c) Jon Danielsson, jond@hag.hi.is" << endl ;
		cout << "\nCommand line has to contain at least filename as the first argument"<<endl;
		cout << "and optionally the following" << endl;
      		cout << "\n\t-n simulations" ;
		cout << "\n\t-i   Number of AGIS iterations, default " << I  ;
		cout << "\n\t-h   do hessian, YES/NO"  ;
		cout << "\n\t-p   Tolerance,  default  " << P ;
		cout << "\n\t-ascale 	Don't autoscale"  ;
		cout << "\n\t-print "  ;
		cout << "\n\t-seed	 Input random seed, default=234.0" << Seed;
		cout << "\n\t-par omega delta nu" ;
		cout << "\n\t-reg does robust and slow regression";

		cout << endl;

	     	exit(1); 
      }      

	strcpy(Fname,argv[1]);

	for(i=1;i<argc; i++)
	{
		if( strcmp(argv[i],"-i")==0 ) I = atoi(argv[i+1]);
		if( strcmp(argv[i],"-n")==0 ) N = atoi(argv[i+1]);
		if( strcmp(argv[i],"-print")==0 ) Print = atoi(argv[i+1]);

		if( strcmp(argv[i],"-h")==0 ){ H = NO; }
		if( strcmp(argv[i],"-p")==0 ) P = atof(argv[i+1]);
		if( strcmp(argv[i],"-ascale")==0 ) Ascale = NO;

		if( strcmp(argv[i],"-z")==0 ) Z = atoi(argv[i+1]);
		if( strcmp(argv[i],"-seed")==0) Seed = atof(argv[i+1]);
		if( strcmp(argv[i],"-reg")==0) Reg = YES;
		if( strcmp(argv[i],"-par")==0) {
			omega = atof(argv[i+1]);		
			delta = atof(argv[i+2]);		
			nu = atof(argv[i+3]);		
		
		};


	}
	head();		// print head
	read_file();
	return (int) NULL;

}

// read input file and print statistics
void command_line::read_file(){

	ifstream in(Fname);
	int num;

	if(!in) {cout << "\nERROR opening file " << Fname << endl;
		err("exit");
	}
	double d;
// first count number of lines in input file
	num=0;
	while(!in.eof() ) {
		in  >> d;
		num++;
	}
	if(num < 200) {
		cout << "\nSV estimation typically requiers at least";
		cout << "\n1000 observations. Change this at own risk";

		err("Not enough observations in input file") ;
	}
	num--;
	T = num;
	in.close();
// allocate memory for data class.  Note two variables are because of autoscale
	data=new double[num+1];
	raw_data=new double[num+1];
	ifstream in2(Fname);
// read input file
	num=1;
	in2  >> d;
	while(!in2.eof() ) {
		raw_data[num]=data[num] = d;
		in2  >> d;
		num++;
	}
	num--;
	in2.close();
// print statistics if requested
	double m1=0.0,m2=0.0,m3=0.0;
	int i;
	for(i=1;i<=num;i++) m1 += data[i];
	m1 /= num;
	for(i=1;i<=num;i++) m2 += (data[i]-m1)*(data[i]-m1);
	m2 /= (num-1);
	if(Print > 0){
		cout << "    Inputfile:  "<<Fname << endl;
		cout << "    Input Data: nobs = " << T << ", mean = " << m1 << ", var = " << m2;
		cout << endl;
	}
}


// print header

void command_line::head(void){
	if(Print >0){
		int i;
		cout << endl;
		for(i=1;i<=80;i++)cout << "_";
		cout << endl;
		cout << "Simulated Maximum Likelihood Estimation\n    v. " 
			<< SVM_VERSION << "  " << SVM_DATE << endl;
		cout << "Stochastic Volatility Model" << endl;
		cout << "Jon Danielsson (c) 1996" << endl;
		cout << "Send comments to: jond@hag.hi.is"<<endl;
/*
cout << "Library calls: " ;
#ifdef NAG_RAND
cout << " NAG";
#endif
#ifdef RANLIB
cout << " RANLIB";
#endif
cout << " random numbers, ";
#ifdef NAG_OPT
cout << " NAG";
#endif
#ifdef SIMPLEX
cout << " SIMPLEX";
#endif
cout << " Optimization" << endl;
*/
		clock();
		cout << endl;
		cout << now();

		cout << "Initial Value:  "  ;
		cout << "omega = " << omega << ", delta " << delta ;
		cout << ", nu = " << nu  << endl;
	}
}


void error(int line , char *file, char *text)
{
        cerr << "\nRun-time error ...." ;
        cerr << "\nLine " << line << " in " << file << " : " << text ;
        cerr << "\n...now exiting to system...\n" << endl;
        flush(cerr);
        exit(1);
}

double var(double *x, int num)
{
        int i;
        double m,v;
        m=v=0.0e0;
        for(i=1;i<=num;i++)
        {
                m += x[i];
                v += x[i]*x[i];
        }
        m /= num;
        v /= num;
        v = (double) num / ( (double) num-1.0 ) * (v - m*m);
        if(v < 0.0e0) v = 0.0e0;
        return v;

}



double eelog(double x ,int l,char *name)
{
        if(x> 0.0e0) return log(x);
        cerr << "\nAttemped log(" << x << ") in line " << l << " in " <<name
<< endl;
        exit(10);
	return 0.0;
}
 
 
double eesqrt(double x ,int l,char *name)
{
        if(x>= 0.0e0) return sqrt(x);
        cerr << "\nAttemped sqrt(" << x << ") in line " << l << " in " <<name << endl;
        exit(10);
	return 0.0;
}
 
double mean(double *x, int num)
{
        int i;
        double m;
        m=0.0e0;
        for(i=1;i<=num;i++) m+= x[i];
        return (m/num);
}

